// --------------------------------------------------------------------
// Copyright (c) 2005 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altrea Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL or Verilog source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions: I2C output data
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author             :| Mod. Date :| Changes Made:
//   V1.0 :| Joe Yang           :| 05/07/10  :|      Initial Revision
// --------------------------------------------------------------------
`define rom_size 6'd9

module CLOCK_500 (
	CLOCK,
	CLOCK_500,
	DATA,
	FINISH,
	GO,
	SW1,
	SW2,
	UPDATE
);
	input  CLOCK;
	input  FINISH;
	input  SW1;
	input  SW2;
	input  UPDATE;
	output CLOCK_500;
	output [23:0]DATA;
	output GO;

reg  [10:0]COUNTER_500;

wire  CLOCK_500=COUNTER_500[9]; //Originalt: [9] Setter frekvensen p SCLK=100kHz

reg  [15:0]ROM[`rom_size:0];
reg  [15:0]DATA_A;
reg  [5:0]address;
wire [23:0]DATA={8'h34,DATA_A};
											
wire GO =((address <= `rom_size) && (FINISH==1))? COUNTER_500[10]:1; //COUNTER_500[10]:1;
		 //if(true) then GO=COUNTER_500[10] else GO=1

always @(posedge UPDATE or posedge FINISH) begin
	if (UPDATE) address=0;
	else if(address <= `rom_size) address=address+1;
	else address = address;
end


reg [7:0]vol = 8'd121; //0dB forsterkning i hodetelefonforst.(121 = 0dB, 30 = -73 dB)


always @(posedge FINISH) begin //Endringer trer i kraft nr en overfring er over
if(SW2 == 0)begin 
	if(SW1 == 0)begin
	//ROM[0]= 16'h1200;		 //Deactive
	ROM[0]= 16'h0c62;	     //power down
	ROM[1]= 16'h0e5b;	     //master,DSP-mode,BCLK not invert
	ROM[2]= 16'h0812;	     //sound select
	
	//ROM[3]= 16'h1001;		 //mclk
	ROM[3]= 16'h100d;		 //Reg.8: mclk 8kHz,USB-mode,BOSR=0(250fs)
	
	ROM[4]= 16'h0017;		 //Left line in
	ROM[5]= 16'h0217;		 //Right line in
	ROM[6]= {8'h04,1'b0,vol[6:0]};		 //
	ROM[7]= {8'h06,1'b0,vol[6:0]};	     //sound vol
	
	//ROM[4]= 16'h1e00;		 //reset	
	ROM[8]= 16'h1200;		 //Deactive
	ROM[`rom_size]= 16'h1201;//active
	DATA_A=ROM[address];
	end	
	
	else if(SW1 == 1)begin
	//ROM[0]= 16'h1200;		 //Deactive
	ROM[0]= 16'h0c62;	     //power down
	ROM[1]= 16'h0e5b;	     //master,DSP-mode,BCLK not invert
	ROM[2]= 16'h0812;	     //sound select
	
	//ROM[3]= 16'h1001;		 //mclk
	ROM[3]= 16'h1019;		 //Reg.8: mclk 32kHz,USB-mode,BOSR=0(250fs)
	
	ROM[4]= 16'h0017;		 //Left line in
	ROM[5]= 16'h0217;		 //Right line in
	ROM[6]= {8'h04,1'b0,vol[6:0]};		 //
	ROM[7]= {8'h06,1'b0,vol[6:0]};	     //sound vol
	
	//ROM[4]= 16'h1e00;		 //reset	
	ROM[8]= 16'h1200;		 //Deactive
	ROM[`rom_size]= 16'h1201;//active
	DATA_A=ROM[address];
	end
end 

else if(SW2 == 1)begin 
	if(SW1 == 0)begin
	//ROM[0]= 16'h1200;		 //Deactive
	ROM[0]= 16'h0c62;	     //power down
	ROM[1]= 16'h0e5b;	     //master,DSP-mode,BCLK not invert
	ROM[2]= 16'h0812;	     //sound select
	
	//ROM[3]= 16'h1001;		 //mclk
	ROM[3]= 16'h1001;		 //Reg.8: mclk 48kHz,USB-mode,BOSR=0(250fs)
	
	ROM[4]= 16'h0017;		 //Left line in
	ROM[5]= 16'h0217;		 //Right line in
	ROM[6]= {8'h04,1'b0,vol[6:0]};		 //
	ROM[7]= {8'h06,1'b0,vol[6:0]};	     //sound vol
	
	//ROM[4]= 16'h1e00;		 //reset	
	ROM[8]= 16'h1200;		 //Deactive
	ROM[`rom_size]= 16'h1201;//active
	DATA_A=ROM[address];
	end	

	else if(SW1 == 1)begin
	//ROM[0]= 16'h1200;		 //Deactive
	ROM[0]= 16'h0c62;	     //power down
	ROM[1]= 16'h0e5b;	     //master,DSP-mode,BCLK not invert
	ROM[2]= 16'h0812;	     //sound select
	
	//ROM[3]= 16'h1001;		 //mclk
	ROM[3]= 16'h101d;		 //Reg.8: mclk 96kHz,USB-mode,BOSR=0(250fs)
	
	ROM[4]= 16'h0017;		 //Left line in
	ROM[5]= 16'h0217;		 //Right line in
	ROM[6]= {8'h04,1'b0,vol[6:0]};		 //
	ROM[7]= {8'h06,1'b0,vol[6:0]};	     //sound vol
	
	//ROM[4]= 16'h1e00;		 //reset	
	ROM[8]= 16'h1200;		 //Deactive
	ROM[`rom_size]= 16'h1201;//active
	DATA_A=ROM[address];
	end	
end
end

always @(posedge CLOCK ) begin
	COUNTER_500=COUNTER_500+1;
end
endmodule