----------------------------------------------------------------------------------
--Audio-buffer. Receives seriel data from ADC, 48 bits including both channels,--- 
--and transmitts 2x24 bit packages,left channel first, in parallel at the output--
----------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity audio_buffer is
port(
	clk,adc_data,lrc,reset,ast_sink_ready : in std_logic;
	filter_data : out std_logic_vector (23 downto 0);
	ast_sink_eop,ast_sink_sop,ast_sink_valid : out std_logic 
);
end entity audio_buffer;

architecture structure of audio_buffer is
type state is (IDLE,RECEIVE,TRANSFER_LEFT,TRANSFER_RIGHT);
signal next_state : state;
signal audio_data : std_logic_vector(47 downto 0);	--Store audio-data, left and right channel
signal counter : integer range 47 downto 0 := 47;
begin
		
	
	FSM : process(clk,reset) 
	begin
	  if(reset = '0') then
	    next_state <= IDLE;
	    filter_data <= (OTHERS => '0');
	    audio_data <= (OTHERS => '0');
	    ast_sink_valid <= '0';
	    ast_sink_eop <= '0';
	    ast_sink_sop <= '0';
	    
		elsif(rising_edge(clk)) then 
			case next_state is
				when IDLE => 
					if(lrc = '1') then --Audio-codec starts sending data from ADC
						ast_sink_eop <= '0'; 
						ast_sink_sop <= '0';
						ast_sink_valid <= '0';
						next_state <= RECEIVE;
					elsif(lrc = '0') then
						ast_sink_eop <= '0';
						ast_sink_sop <= '0';
						ast_sink_valid <= '0';
						next_state <= IDLE;
					end if;
					
				when RECEIVE => --Receive data from both channels, left channel first
					if(counter > 0) then
						audio_data(counter) <= adc_data; --Saves MSB for left channel on audio_data[47], MSB for right channel at audio_data[23]
						counter <= counter - 1;
						next_state <= RECEIVE;
					else
						audio_data(counter) <= adc_data;
						counter <= 47;
						next_state <= TRANSFER_LEFT;
					end if;
					
		
				when TRANSFER_LEFT => --Puts left channel data on the parallel output
					if(ast_sink_ready = '1') then --If FIR filter is ready to accept data
						ast_sink_sop <= '1'; --Start of data package
						filter_data(23 downto 0) <= audio_data(47 downto 24); --Left channel, bit 0 = MSB
						ast_sink_valid <= '1';
						next_state <= TRANSFER_RIGHT;
					else --Wait until FIR filter is ready to accept data
						next_state <= TRANSFER_LEFT;
					end if;

				when TRANSFER_RIGHT => --Puts right channel data on the parallel output
					ast_sink_sop <= '0'; 
					ast_sink_eop <= '1'; --End of data package
					filter_data(23 downto 0) <= audio_data(23 downto 0); --Right channel, bit 24 = MSB
					next_state <= IDLE;
			end case;
		end if;
	end process;
		
end architecture;