-------------------------------------------------------------------------------
--Audio output buffer. Receives 2 x 24 bit parallel data packages from filter--
--and puts 48 bits of serial data to the output when DAC is ready to receive.--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity audio_buffer_out is
port(
	clk,lrc,reset,ast_source_valid : in std_logic;
	audio_data_in : in std_logic_vector(23 downto 0);
	dac_data : out std_logic
);
end entity audio_buffer_out;

architecture structure of audio_buffer_out is
type state is (IDLE,RECEIVE_RIGHT,START_TRANSFER,CONT_TRANSFER);
signal next_state : state;
signal audio_data : std_logic_vector(47 downto 0);
signal counter : integer range 47 downto 0 := 47;
begin

	FSM : process(clk,reset) 
	begin
		if(reset = '0') then
		  next_state <= IDLE;
		  dac_data <= '0';
		  audio_data(47 downto 0) <= (OTHERS => '0');
		  
		elsif(rising_edge(clk)) then
			case next_state is
				when IDLE =>
					audio_data(47 downto 24) <= audio_data_in(23 downto 0); --Saves left ch.data from filter on det 24 MSBs
					if(ast_source_valid = '1') then --If data from FIR is ready
						dac_data <= '0';
						next_state <= RECEIVE_RIGHT;
					else
						dac_data <= '0';
						next_state <= IDLE;
					end if;
					
				when RECEIVE_RIGHT => --Receive and save data (24 bit) for right channel
					audio_data(23 downto 0) <= audio_data_in(23 downto 0);
					next_state <= START_TRANSFER;
									
				when START_TRANSFER => --Transmit data
					if(lrc = '1') then --When DAC is ready
						dac_data <= audio_data(counter); --MSB left channel
						counter <= counter - 1;
						next_state <= CONT_TRANSFER;
					elsif(lrc = '0') then 
						next_state <= START_TRANSFER;
					end if;
					
				when CONT_TRANSFER => --Sends audio data to the DAC, 48 bits serial
					if(counter > 0) then
						dac_data <= audio_data(counter);
						counter <= counter - 1; 
						next_state <= CONT_TRANSFER;
					else
						dac_data <= audio_data(counter);
						counter <= 47;
						next_state <= IDLE;
					end if;
			end case;
		end if;
	end process;
		
end architecture;